from StringIO import StringIO
from Products.CMFCore.utils import getToolByName

from Products.PloneSoftwareCenter import config

EXTENSION_PROFILES = ('Products.PloneSoftwareCenter:default',)

def install(self):
    out = StringIO()

    tool=getToolByName(self, "portal_setup")

    try:
        tool.runAllImportStepsFromProfile(
            'profile-%s' % EXTENSION_PROFILES[0],
            purge_old=False)
    except AttributeError:   # before plone 3
        tool.setImportContext('profile-%s' % EXTENSION_PROFILES[0])
        tool.runAllImportSteps(purge_old=False)

    print >> out, "Successfully installed %s" % config.PROJECTNAME

    return out.getvalue()


### leaving the old cruft in place for now
### only renaming (re)install not to confude the quickintaller

from Products.Archetypes.atapi import listTypes
from Products.Archetypes.Extensions.utils import installTypes
from Products.Archetypes.Extensions.utils import install_subskin

# Get the custom workflows

## from Products.PloneSoftwareCenter.Extensions import AreaWorkflow
## from Products.PloneSoftwareCenter.Extensions import ImprovementProposalWorkflow
## from Products.PloneSoftwareCenter.Extensions import PackageWorkflow
## from Products.PloneSoftwareCenter.Extensions import ReleaseWorkflow

import transaction

def setWorkflowForType(self, out, tool, type, workflowName):
    """Set the workflow for a given type."""
    tool.setChainForPortalTypes((type,), workflowName)
    print >> out, 'Set workflow for %s to %s.' % (type, workflowName)

def addWorkflow(self, out, tool, workflowName, workflowId):
    """Add the given workflow.

    The workflowName must be the exact name set in the workflow,
    which in turn must have been registered with the workflows factory.
    """
    if not workflowId in tool.objectIds():
        tool.manage_addWorkflow(workflowName, workflowId)
        print >> out, 'Added workflow %s.' % workflowName

def addWorkflowScript(self, out, tool, wfname, id, title, module, method):
    """Add the given external method to the given workflow as a script"""
    
    wf = tool[wfname]
    
    if not id in wf.scripts.objectIds():
        factory = wf.scripts.manage_addProduct['ExternalMethod']
        factory.manage_addExternalMethod(id, title, module, method)

        print >> out, "Added external method %s.%s to workflow %s" % \
                          (module, method, wfname)

def addCatalogIndex(self, out, catalog, index, type, extra=None):
    """Add the given index name, of the given type, to the catalog."""
    if index not in catalog.indexes():
        catalog.addIndex(index, type, extra)
        print >> out, "Added index", index, "to catalog"
    else:
        print >> out, "Index", index, "already in catalog"

def addCatalogMetadata(self, out, catalog, column):
    """Add the given column to the catalog's metadata schema"""
    if column not in catalog.schema():
        catalog.addColumn(column)
        print >> out, "Added", column, "to catalog metadata"
    else:
        print >> out, column, "already in catalog metadata"

def removeCatalogMetadata(self, out, catalog, column):
    """Delete the given metadata column"""
    # QueueCatalog work-around
    if catalog.meta_type == 'ZCatalog Queue':
         catalog = self.unrestrictedTraverse(catalog._location)
    if column in catalog.schema():
        catalog.delColumn(column)
        print >> out, "Removed column", column
    else:
        print >> out, "Column", column, "not in catalog"

def installWorkflows(self, out, reinstall):
    workflow = getToolByName(self, 'portal_workflow')
    print >> out, 'Installing custom workflows'

    # Register the workflows
    AreaWorkflow.install()
    ImprovementProposalWorkflow.install()
    PackageWorkflow.install()
    ReleaseWorkflow.install()

    addWorkflow(self, out, workflow,
                   'psc_area_workflow (PSC Area workflow)',
                   'psc_area_workflow')
    addWorkflow(self, out, workflow,
                   'psc_improvementproposal_workflow (PSC Improvement Proposal workflow)',
                   'psc_improvementproposal_workflow')
    addWorkflow(self, out, workflow,
                   'psc_package_workflow (PSC Project workflow)',
                   'psc_package_workflow')
    addWorkflow(self, out, workflow,
                   'psc_release_workflow (PSC Release workflow)',
                   'psc_release_workflow')

    # Enable the workflow for the types
    setWorkflowForType(self, out, workflow,
                           'PSCProject',
                           'psc_package_workflow')
    setWorkflowForType(self, out, workflow,
                           'PloneSoftwareCenter',
                           'psc_area_workflow')
    setWorkflowForType(self, out, workflow,
                           'PSCFile',
                           '')
    setWorkflowForType(self, out, workflow,
                           'PSCFileLink',
                           '')
    setWorkflowForType(self, out, workflow,
                           'PSCImprovementProposal',
                           'psc_improvementproposal_workflow')
    setWorkflowForType(self, out, workflow,
                           'PSCImprovementProposalFolder',
                           '')
    setWorkflowForType(self, out, workflow,
                           'PSCRelease',
                           'psc_release_workflow')
    setWorkflowForType(self, out, workflow,
                           'PSCReleaseFolder',
                           '')
    setWorkflowForType(self, out, workflow,
                           'PSCDocumentationFolder',
                           '')
    
    try:
        workflow.updateRoleMappings()
    except:
        pass

    # Add scripts
    addWorkflowScript(self, out, workflow, 'psc_release_workflow',
                        id = 'release_new_state',
                        title = 'Make a release into a new state',
                        module = 'PloneSoftwareCenter.WorkflowScripts',
                        method = 'releaseNewState')
    addWorkflowScript(self, out, workflow, 'psc_release_workflow',
                        id = 're_release_state',
                        title = 'Make a new release into the same state',
                        module = 'PloneSoftwareCenter.WorkflowScripts',
                        method = 'reReleaseState')
    addWorkflowScript(self, out, workflow, 'psc_package_workflow',
                        id = 'give_reviewer_localrole',
                        title = 'Give reviewer localrole',
                        module = 'PloneSoftwareCenter.WorkflowScripts',
                        method = 'giveReviewerLocalrole')
    addWorkflowScript(self, out, workflow, 'psc_package_workflow',
                        id = 'take_reviewer_localrole',
                        title = 'Take reviewer localrole',
                        module = 'PloneSoftwareCenter.WorkflowScripts',
                        method = 'takeReviewerLocalrole')
                        
    print >> out, 'Done installing workflows.'

def addPortalFactoryType(self, out, factory, metaType):
    """Add the given type to the list of types used by PortalFactory"""
    
    types = factory.getFactoryTypes().keys()
    types.append(metaType)
    factory.manage_setPortalFactoryTypes(listOfTypeIds = types)
    
    print >> out, "Added %s to portal_factory" % (metaType,)

def installFactoryTypes(self, out, reinstall):
    """Add all types to factory"""
    
    factory = getToolByName(self, 'portal_factory')
    
    for t in ('PloneSoftwareCenter', 'PSCProject', 'PSCReleaseFolder', 
                'PSCImprovementProposalFolder', 'PSCRelease', 
                'PSCImprovementProposal', 'PSCFile', 'PSCFileLink',
                'PSCDocumentationFolder',):
        addPortalFactoryType(self, out, factory, t)

def satisfyDependencies(self, out):
    qi = getToolByName(self, 'portal_quickinstaller')
    def installProduct(product, hard):
        try:
            qi.installProduct(product)
            transaction.savepoint(1)
        except AttributeError:
            msg = 'Failed to install %s.' % dep
            if hard:
                raise ValueError, msg
            else:
                print >> out, msg
        else:
            print >> out, 'Installed dependency %s.' % dep

    for dep in config.HARD_DEPS:
        installProduct(dep, hard=True)
    for dep in config.SOFT_DEPS:
        installProduct(dep, hard=False)

def setupCatalog(self, out, reinstall):
    catalog = getToolByName(self, 'portal_catalog')
    addCatalogMetadata(self, out, catalog, 'getCategoryTitles')
    addCatalogMetadata(self, out, catalog, 'UID')
    addCatalogIndex(self, out, catalog, 'releaseCount', 'FieldIndex')
    
def setupNavtree(self, out, reinstall):
    portal_properties = getToolByName(self, 'portal_properties')
    navtree_properties = getattr(portal_properties, 'navtree_properties')
    parentMetaTypesNotToQuery = list(navtree_properties.getProperty('parentMetaTypesNotToQuery'))
    for t in ('PloneSoftwareCenter', 'PSCReleaseFolder', 'PSCImprovementProposalFolder', 'PSCRelease'):
        if t not in parentMetaTypesNotToQuery:
            parentMetaTypesNotToQuery.append(t)
    navtree_properties._updateProperty('parentMetaTypesNotToQuery', parentMetaTypesNotToQuery)
    
def cleanCatalog(self, out, reinstall):
    catalog = getToolByName(self, 'portal_catalog')
    if not reinstall:
        removeCatalogMetadata(self, out, catalog, 'getCategoryTitles')

def addFolderContentsViewTypes(self, out, reinstall):
    portal_properties = getToolByName(self, 'portal_properties')
    site_properties = getattr(portal_properties, 'site_properties', None)
    if site_properties is not None:
        useView = list(site_properties.typesUseViewActionInListings)
        if 'PSCFile' not in useView:
            useView.append('PSCFile')
        site_properties._updateProperty('typesUseViewActionInListings', useView)

def old_install(self, reinstall=False):
    out = StringIO()

    satisfyDependencies(self, out)

    install_subskin(self, out, config.GLOBALS)
    installTypes(self, out, listTypes(config.PROJECTNAME), config.PROJECTNAME)

    migrateFTIs(self, product=config.PROJECTNAME)

    installFactoryTypes(self, out, reinstall)
    installWorkflows(self, out, reinstall)
    setupCatalog(self, out, reinstall)
    addFolderContentsViewTypes(self, out, reinstall)
    setupNavtree(self, out, reinstall)

    print >> out, 'Successfully installed %s.' % config.PROJECTNAME

    return out.getvalue()

def old_uninstall(self, reinstall=False):
    out = StringIO()

    cleanCatalog(self, out, reinstall)

    print >> out, 'Successfully uninstalled %s.' % config.PROJECTNAME

    return out.getvalue()
